
#define protected public
#include <qobject.h>
#undef protected
#include <qptrlist.h>
#include <kxmlguiclient.h>
#include <kxmlguifactory.h>
#include <kaction.h>
#include <kinstance.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kgenericfactory.h>
#include <kdebug.h>

#include "plugin_usagemonitor.h"
#include <plugin_usagemonitor.moc>

typedef KGenericFactory<PluginUsageMonitor> UsageMonitorFactory;
K_EXPORT_COMPONENT_FACTORY( libusagemonitorplugin, 
                            UsageMonitorFactory( "usagemonitor" ) );

PluginUsageMonitor::PluginUsageMonitor( QObject* parent, const char* name,
                                      const QStringList & /*args*/ )
    : Plugin( parent, name )
{
    // Instantiate all of your actions here.  These will appear in
    // Konqueror's menu and toolbars.
    KToggleAction *mon = new KToggleAction( i18n("&Usage Monitor"), "usagemonitor", 0,
					    actionCollection(), "plugin_usagemonitor" );
    connect( mon, SIGNAL(toggled(bool)), SLOT(setMonitor(bool)));
}

PluginUsageMonitor::~PluginUsageMonitor()
{
}

void PluginUsageMonitor::setMonitor( bool yes )
{
    KXMLGUIClient *part = dynamic_cast<KXMLGUIClient *>(parent());

    if ( !part ) {
        QString title( i18n( "Cannot Monitor Source" ) );
        QString text( i18n( "The parent of this plugin is not a KXMLGUIClient." ) );

        KMessageBox::sorry( 0, text, title );
        return;
    }

    if ( yes ) {
	QPtrList<KXMLGUIClient> clients = part->factory()->clients();
	KXMLGUIClient *c;
	for ( c = clients.first(); c; c = clients.next() ) {
	    if ( yes )
		addClient(c);
	    else
		delClient(c);
	}

 	connect( part->factory(), SIGNAL(clientAdded(KXMLGUIClient *)),
 		 this, SLOT(addClient(KXMLGUIClient *)) );
 	connect( part->factory(), SIGNAL(clientRemoved(KXMLGUIClient *)),
 		 this, SLOT(delClient(KXMLGUIClient *)) );
    }
    else {
	delClient(part);
    }
}

void PluginUsageMonitor::addClient( KXMLGUIClient *c )
{
//    kdDebug() << "Monitor:\t" << "addClient() " << c->xmlFile() << endl;
    KActionCollection *ac = c->actionCollection();
    addActionCollection( ac );
}

void PluginUsageMonitor::delClient( KXMLGUIClient *c )
{
//    kdDebug() << "Monitor:\t" << "delClient() " << c->xmlFile() << endl;
    KActionCollection *ac = c->actionCollection();
    delActionCollection( ac );
}

void PluginUsageMonitor::addActionCollection( KActionCollection *ac )
{
//    kdDebug() << "Monitor:\t" << "addActionCollection() " << ac->name() << endl;
    connect( ac, SIGNAL(inserted(KAction *)), this, SLOT(addAction(KAction *)) );
    connect( ac, SIGNAL(removed(KAction *)), this, SLOT(delAction(KAction *)) );

    KActionPtrList actions = ac->actions();

    KActionPtrList::iterator it;
    for ( it = actions.begin(); it != actions.end(); ++it ) {
	connect( (*it), SIGNAL(activated()), this, SLOT(slotLogAction()) );
    }
}

void PluginUsageMonitor::delActionCollection( KActionCollection *ac )
{
//kdDebug() << "Monitor:\t" << "delActionCollection() " << ac->name() << endl;
    disconnect( ac, SIGNAL(inserted(KAction *)), this, SLOT(addAction(KAction *)) );
    disconnect( ac, SIGNAL(removed(KAction *)), this, SLOT(delAction(KAction *)) );

    KActionPtrList actions = ac->actions();

    KActionPtrList::iterator it;
    for ( it = actions.begin(); it != actions.end(); ++it ) {
	disconnect( (*it), SIGNAL(activated()), this, SLOT(slotLogAction()) );
    }
}

void PluginUsageMonitor::addAction( KAction *a )
{
//    kdDebug() << "Monitor:\t" << "addAction() " << a->name() << endl;
    connect( a, SIGNAL(activated()), this, SLOT(slotLogAction()) );
}

void PluginUsageMonitor::delAction( KAction *a )
{
//    kdDebug() << "Monitor:\t" << "delAction() " << a->name() << endl;
    disconnect( a, SIGNAL(activated()), this, SLOT(slotLogAction()) );
}

void PluginUsageMonitor::slotLogAction()
{
    QObject *snd = (QObject *) sender();
    const char *name = snd ? snd->name() : "None";
    const QObject *clr = snd ? snd->sender() : 0;
    const char *callerclazz = clr ? clr->className() : "None";
    const char *calledby = clr ? clr->name() : "None";

    kdDebug() << "Monitor:\t" << name << "\t" << calledby << "\t" << callerclazz << endl;
}
